@@ -1,3 +1,27 @@
+############################################################################
+#
+# Copyright (C) 2016 The Qt Company Ltd.
+# Contact: https://www.qt.io/licensing/
+#
+# This file is part of Qt Creator.
+#
+# Commercial License Usage
+# Licensees holding valid commercial Qt licenses may use this file in
+# accordance with the commercial license agreement provided with the
+# Software or, alternatively, in accordance with the terms contained in
+# a written agreement between you and The Qt Company. For licensing terms
+# and conditions see https://www.qt.io/terms-conditions. For further
+# information use the contact form at https://www.qt.io/contact-us.
+#
+# GNU General Public License Usage
+# Alternatively, this file may be used under the terms of the GNU
+# General Public License version 3 as published by the Free Software
+# Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
+# included in the packaging of this file. Please review the following
+# information to ensure the GNU General Public License requirements will
+# be met: https://www.gnu.org/licenses/gpl-3.0.html.
+#
+############################################################################
 
 try:
     import __builtin__
@@ -144,13 +168,15 @@ class PlainDumper:
         self.typeCache = {}
 
     def __call__(self, d, value):
-        printer = self.printer.invoke(value)
+        printer = self.printer.gen_printer(value)
         lister = getattr(printer, "children", None)
         children = [] if lister is None else list(lister())
         d.putType(self.printer.name)
         val = printer.to_string()
         if isinstance(val, str):
             d.putValue(val)
+        elif sys.version_info[0] <= 2 and isinstance(val, unicode):
+            d.putValue(val)
         else: # Assuming LazyString
             d.putCharArrayHelper(val.address, val.length, val.type.sizeof)
 
@@ -254,11 +280,11 @@ class Dumper(DumperBase):
         self.useFancy = int(args.get("fancy", "0"))
         self.forceQtNamespace = int(args.get("forcens", "0"))
         self.passExceptions = int(args.get("passexceptions", "0"))
+        self.showQObjectNames = int(args.get("qobjectnames", "0"))
         self.nativeMixed = int(args.get("nativemixed", "0"))
         self.autoDerefPointers = int(args.get("autoderef", "0"))
         self.partialUpdate = int(args.get("partial", "0"))
         self.fallbackQtVersion = 0x50200
-        self.sortStructMembers = bool(args.get("sortstructs", True))
 
         #warn("NAMESPACE: '%s'" % self.qtNamespace())
         #warn("EXPANDED INAMES: %s" % self.expandedINames)
@@ -353,7 +379,13 @@ class Dumper(DumperBase):
     def canCallLocale(self):
         return False if self.is32bit() else True
 
+    def reportTime(self, hint):
+        #from datetime import datetime
+        #warn("%s: %s" % (hint, datetime.now().time().isoformat()))
+        pass
+
     def fetchVariables(self, args):
+        self.reportTime("begin fetch")
         self.prepare(args)
         partialVariable = args.get("partialvar", "")
         isPartial = len(partialVariable) > 0
@@ -389,6 +421,8 @@ class Dumper(DumperBase):
         else:
             locals = self.listOfLocals()
 
+        self.reportTime("locals")
+
         # Take care of the return value of the last function call.
         if len(self.resultVarName) > 0:
             try:
@@ -401,7 +435,6 @@ class Dumper(DumperBase):
                 # Don't bother. It's only supplementary information anyway.
                 pass
 
-        locals.sort(key = lambda item: item.name)
         for item in locals:
             value = self.downcast(item.value) if self.useDynamicType else item.value
             with OutputSafer(self):
@@ -438,7 +471,9 @@ class Dumper(DumperBase):
 
         self.output.append(',partial="%d"' % isPartial)
 
+        self.reportTime("before print: %s" % len(self.output))
         safePrint(''.join(self.output))
+        self.reportTime("after print")
 
     def enterSubItem(self, item):
         if not item.iname:
@@ -463,7 +498,7 @@ class Dumper(DumperBase):
             if self.passExceptions:
                 showException("SUBITEM", exType, exValue, exTraceBack)
             self.putNumChild(0)
-            self.putSpecialValue(SpecialNotAccessibleValue)
+            self.putSpecialValue("notaccessible")
         try:
             if self.currentType.value:
                 typeName = self.stripClassTag(self.currentType.value)
@@ -471,11 +506,10 @@ class Dumper(DumperBase):
                     self.put('type="%s",' % typeName) # str(type.unqualified()) ?
 
             if  self.currentValue.value is None:
-                self.put('value="",encoding="%d",numchild="0",'
-                        % SpecialNotAccessibleValue)
+                self.put('value="",encoding="notaccessible",numchild="0",')
             else:
                 if not self.currentValue.encoding is None:
-                    self.put('valueencoded="%d",' % self.currentValue.encoding)
+                    self.put('valueencoded="%s",' % self.currentValue.encoding)
                 if self.currentValue.elided:
                     self.put('valueelided="%d",' % self.currentValue.elided)
                 self.put('value="%s",' % self.currentValue.value)
@@ -526,13 +560,13 @@ class Dumper(DumperBase):
         except:
             return None
 
-    def isBadPointer(self, value):
+    def pointerInfo(self, value):
         try:
             target = value.dereference()
             target.is_optimized_out # Access test.
-            return False
+            return (True, toInteger(value))
         except:
-            return True
+            return (False, toInteger(value))
 
     def makeValue(self, typeobj, init):
         typename = "::" + self.stripClassTag(str(typeobj));
@@ -897,31 +931,14 @@ class Dumper(DumperBase):
     def simpleEncoding(self, typeobj):
         code = typeobj.code
         if code == BoolCode or code == CharCode:
-            return Hex2EncodedInt1
+            return "int:1"
         if code == IntCode:
             if str(typeobj).find("unsigned") >= 0:
-                if typeobj.sizeof == 1:
-                    return Hex2EncodedUInt1
-                if typeobj.sizeof == 2:
-                    return Hex2EncodedUInt2
-                if typeobj.sizeof == 4:
-                    return Hex2EncodedUInt4
-                if typeobj.sizeof == 8:
-                    return Hex2EncodedUInt8
+                 return "uint:%d" % typeobj.sizeof
             else:
-                if typeobj.sizeof == 1:
-                    return Hex2EncodedInt1
-                if typeobj.sizeof == 2:
-                    return Hex2EncodedInt2
-                if typeobj.sizeof == 4:
-                    return Hex2EncodedInt4
-                if typeobj.sizeof == 8:
-                    return Hex2EncodedInt8
+                 return "int:%d" % typeobj.sizeof
         if code == FloatCode:
-            if typeobj.sizeof == 4:
-                return Hex2EncodedFloat4
-            if typeobj.sizeof == 8:
-                return Hex2EncodedFloat8
+            return "float:%d" % typeobj.sizeof
         return None
 
     def isReferenceType(self, typeobj):
@@ -937,7 +954,7 @@ class Dumper(DumperBase):
         if value is None:
             # Happens for non-available watchers in gdb versions that
             # need to use gdb.execute instead of gdb.parse_and_eval
-            self.putSpecialValue(SpecialNotAvailableValue)
+            self.putSpecialValue("notaccessible")
             self.putType("<unknown>")
             self.putNumChild(0)
             return
@@ -946,7 +963,7 @@ class Dumper(DumperBase):
         typeName = str(typeobj)
 
         if value.is_optimized_out:
-            self.putSpecialValue(SpecialOptimizedOutValue)
+            self.putSpecialValue("optimizedout")
             self.putType(typeName)
             self.putNumChild(0)
             return
@@ -967,7 +984,7 @@ class Dumper(DumperBase):
             try:
                 # Try to recognize null references explicitly.
                 if toInteger(value.address) == 0:
-                    self.putSpecialValue(SpecialNullReferenceValue)
+                    self.putSpecialValue("nullreference")
                     self.putType(typeName)
                     self.putNumChild(0)
                     return
@@ -995,7 +1012,7 @@ class Dumper(DumperBase):
                 self.putBetterType("%s &" % self.currentType.value)
                 return
             except RuntimeError:
-                self.putSpecialValue(SpecialOptimizedOutValue)
+                self.putSpecialValue("optimizedout")
                 self.putType(typeName)
                 self.putNumChild(0)
                 return
@@ -1076,7 +1093,7 @@ class Dumper(DumperBase):
             # Anonymous union. We need a dummy name to distinguish
             # multiple anonymous unions in the struct.
             self.putType(typeobj)
-            self.putSpecialValue(SpecialEmptyStructureValue)
+            self.putSpecialValue("emptystructure")
             self.anonNumber += 1
             with Children(self, 1):
                 self.listAnonymous(value, "#%d" % self.anonNumber, typeobj)
@@ -1086,7 +1103,7 @@ class Dumper(DumperBase):
             # FORTRAN strings
             size = typeobj.sizeof
             data = self.readMemory(value.address, size)
-            self.putValue(data, Hex2EncodedLatin1, 1)
+            self.putValue(data, "latin1", 1)
             self.putType(typeobj)
 
         if typeobj.code != StructCode and typeobj.code != UnionCode:
@@ -1115,17 +1132,21 @@ class Dumper(DumperBase):
         #warn("INAME: %s " % self.currentIName)
         #warn("INAMES: %s " % self.expandedINames)
         #warn("EXPANDED: %s " % (self.currentIName in self.expandedINames))
-        staticMetaObject = self.extractStaticMetaObject(value.type)
-        if staticMetaObject:
-            self.putQObjectNameValue(value)
+        if self.showQObjectNames:
+            staticMetaObject = self.extractStaticMetaObject(value.type)
+            if staticMetaObject:
+                self.putQObjectNameValue(value)
         self.putType(typeName)
         self.putEmptyValue()
         self.putNumChild(len(typeobj.fields()))
 
         if self.currentIName in self.expandedINames:
             innerType = None
+            self.put('sortable="1"')
             with Children(self, 1, childType=innerType):
                 self.putFields(value)
+                if not self.showQObjectNames:
+                    staticMetaObject = self.extractStaticMetaObject(value.type)
                 if staticMetaObject:
                     self.putQObjectGuts(value, staticMetaObject)
 
@@ -1193,15 +1214,6 @@ class Dumper(DumperBase):
 
     def putFields(self, value, dumpBase = True):
             fields = value.type.fields()
-            if self.sortStructMembers:
-                def sortOrder(field):
-                    if field.is_base_class:
-                        return 0
-                    if field.name and field.name.startswith("_vptr."):
-                        return 1
-                    return 2
-                fields.sort(key = lambda field: "%d%s" % (sortOrder(field), field.name))
-
             #warn("TYPE: %s" % value.type)
             #warn("FIELDS: %s" % fields)
             baseNumber = 0
@@ -1235,6 +1247,7 @@ class Dumper(DumperBase):
                         # int (**)(void)
                         n = 100
                         self.putType(" ")
+                        self.put('sortgroup="1"')
                         self.putValue(value[field.name])
                         self.putNumChild(n)
                         if self.isExpanded():
@@ -1258,6 +1271,7 @@ class Dumper(DumperBase):
                         baseNumber += 1
                         with UnnamedSubItem(self, "@%d" % baseNumber):
                             baseValue = value.cast(field.type)
+                            self.put('sortgroup="2"')
                             self.putBaseClassName(field.name)
                             self.putAddress(baseValue.address)
                             self.putItem(baseValue, False)
@@ -1332,8 +1346,7 @@ class Dumper(DumperBase):
     #                         s = self.readMemory(data, 2 * size)
     #
     #                thread = gdb.selected_thread()
-    #                inner = '{valueencoded="';
-    #                inner += str(Hex4EncodedLittleEndianWithoutQuotes)+'",id="'
+    #                inner = '{valueencoded="uf16:2:0",id="'
     #                inner += str(thread.num) + '",value="'
     #                inner += s
     #                #inner += self.encodeString(objectName)
@@ -1744,7 +1757,7 @@ class CliDumper(Dumper):
             if self.passExceptions:
                 showException("SUBITEM", exType, exValue, exTraceBack)
             self.putNumChild(0)
-            self.putSpecialValue(SpecialNotAccessibleValue)
+            self.putSpecialValue("notaccessible")
         try:
             if self.currentType.value:
                 typeName = self.stripClassTag(self.currentType.value)
@@ -1754,11 +1767,11 @@ class CliDumper(Dumper):
                 self.put('<not accessible>')
             else:
                 value = self.currentValue.value
-                if self.currentValue.encoding is Hex2EncodedLatin1:
+                if self.currentValue.encoding == "latin1":
                     value = self.hexdecode(value)
-                elif self.currentValue.encoding is Hex2EncodedUtf8:
+                elif self.currentValue.encoding == "utf8":
                     value = self.hexdecode(value)
-                elif self.currentValue.encoding is Hex4EncodedLittleEndian:
+                elif self.currentValue.encoding == "utf16":
                     b = bytes.fromhex(value)
                     value = codecs.decode(b, 'utf-16')
                 self.put('"%s"' % value)
@@ -1801,6 +1814,7 @@ class CliDumper(Dumper):
         args['fancy'] = 1
         args['passexception'] = 1
         args['autoderef'] = 1
+        args['qobjectnames'] = 1
         name = args['varlist']
         self.prepare(args)
         self.output = name + ' = '
